/**---------------------------------------------------------------------------

  Name         :  main.c
  Description  :  XMC2Go based logic analyzer
  Author       :  Aleksander Borysiuk

  History      :  2014.07.28 - ver.1.0

------------------------------------------------------------------------------*/

/**----------------------------------------------------------------------------
  Info:
  This project implements 8 input logic analyzer on XMC1100 microcontroller.
  Logic analyzer obtains the following features:
    - number of inputs:   8 inputs in 1 group
    - device interface:   serial
    - device clock speed: 32MHz
    - capture clock:      internal
    - sample rates:       10Hz, 20hz, 50Hz, 100Hz, 200Hz, 500Hz, 1kHz, 2kHz, 5kHz,
                          10kHz, 20kHz, 50kHz, 100kHz, 200kHz, 500kHz
    - sample buffer size: 64B, 128B, 256B, 512B, 1024B, 2048B, 4096B, 8192B
    - triggers:           parallel, 1 stage

  The following I/O MCU ports are used as logic analyzer inputs:
      P0.0  : input 5
      P0.5  : input 0
      P0.6  : input 1
      P0.7  : input 2
      P0.8  : input 3
      P0.9  : input 4
      P0.14 : input 6
      P0.15 : input 7

  Logic analyzer supports SUMP communication protocol and may be used with any
  SUMP compatible client (e.g. OpenBench LogicSniffer). The following SUMP client
  features are implemented:
    - device identification,
    - device metadata,
    - device reset,
    - device run,
    - sampling frequency programming (sampling frequency divider programming),
    - trigger mask and value programming (for stage 0),
    - read and delay count programming.

  This project runs without any modifications on XMC 2Go kit. It uses kit's debug
  IC as RS to USB converter.

------------------------------------------------------------------------------*/


/* Includes ------------------------------------------------------------------*/
#include "XMC1100.h"
#include "GPIO.h"
#include "Sump.h"

/* Private define ------------------------------------------------------------*/
#define SYS_TICK_FRQ    10             // SysTicks are generated 10 times per second
                                       // (i.e. every 100ms)
#define LED_EVENT       10             // LED1 status updated every second

// UART baud rate constants for 115.2kbps @ MCLK=32MHz
//    BDR = (fPB=32MHz) * STEP/1024 * 1/(PDIV+1) * 1/(DCTQ+1) * 1/(PCTQ+1)
#define UART_FDR_STEP   590UL          // fractional divider division step
#define UART_BRG_PDIV   9UL            // divider factor to generate fPDIV
#define UART_BRG_DCTQ   15UL           // denominator for time quanta counter
#define UART_BRG_PCTQ   0UL            // pre-divider for time quanta counter

#define RX_TIMEOUT      5              // UART receive data timeout (x100ms)

#define SAMPLE_BUF_SIZE 8192           // sample buffer size in bytes (i.e. 8kB)
                                       // sample buffer size always equals to 2^n !!!

#define OFF             0
#define ON              1

/* Private macro -------------------------------------------------------------*/
// XMC 2Go LED controls
#define LED1_ON         P1_0_set()
#define LED1_OFF        P1_0_reset()
#define LED1_TOGGLE     P1_0_toggle()
#define LED2_ON         P1_1_set()
#define LED2_OFF        P1_1_reset()
#define LED2_TOGGLE     P1_1_toggle()

// UART putchar macro
#define UART_PUTCH(char) USIC0_CH0->IN[0] = char

/* Private variables ---------------------------------------------------------*/
// Logic analyzer id data
// - standard id: SLA, protocol ver.1
const uint8_t sla_stdID[] = {'1','A','L','S'};
// - meta data: device name
const uint8_t sla_devName[] = {MDATA_DEV_NAME, 'X','M','C','2','G','O',' ','L','A',' ','v','1','.','0','\0'};
// - meta data: version of the main firmware
const uint8_t sla_devVer[] = {MDATA_FPGA_VER, '1','.','0','\0'};
// - meta data: available sample memory in bytes = 0x00002000 (i.e. 8192dec)
const uint8_t sla_memSize[5] = {MDATA_SMPL_MEM, 0x00, 0x00, 0x20, 0x00};
// - meta data: maksimum sample rate in Hz =  0x0007A120 (i.e. 500000dec)
const uint8_t sla_smplRate[5] = {MDATA_MAX_FRQ, 0x00, 0x07, 0xA1, 0x20};
// - meta data: short number of usable probes = 8
const uint8_t sla_probesQtty[2] = {MDATA_PROBES_SHRT, 0x08};
// - meta data: short protocol version = 2
const uint8_t sla_protVer[2] = {MDATA_PROTOCOL_SHRT, 0x02};

__IO uint8_t rx_timer;                 // UART receiver timeout timer

struct {
  uint8_t len;                         // buffer data length
  uint8_t data[5];                     // command code and data
} cmd_buf;                             // receive command buffer

struct {
  uint32_t trig_cntr;                  // after trigger samples counter
  uint8_t  trig_value;                 // trigger value
  uint8_t  trig_mask;                  // trigger mask
  uint16_t wr_idx;                     // buffer write index
  uint8_t  data[SAMPLE_BUF_SIZE];      // input samples
} sample_buf;                          // sampling buffer

struct {
  uint8_t  state;                      // transmitter state
  uint32_t cntr;                       // read samples counter
} sample_tx;                           // samples rs transmitter

/* Private function prototypes -----------------------------------------------*/
// - functions
void UART_Init(void);
void UART_SendData(uint8_t *p_data, uint16_t len);
void SamplingTimer_Init(void);
void SUMP_ShortCmd(void);
void SUMP_LongCmd(void);
// - IRQ handlers
//  (sampling timer handler runs from SRAM for maximum speed)
void SysTick_Handler(void);
void USIC0_0_IRQHandler(void);
void CCU40_0_IRQHandler(void) __attribute__((section(".ISRAMCode")));

/* Private functions ---------------------------------------------------------*/

int main(void)
{
  // System clock is already set by SystemInit() function and running

  // Logic Analyzer inputs configuration
  // (used inptus: pull-down, unused inputs: pull-up)
  PORT0->IOCR0 = (INPUT_PU << 24) | (INPUT_PU << 16) | (INPUT_PU << 8) | INPUT_PD; // P0.3_P0.0
  PORT0->IOCR4 = (INPUT_PD << 24) | (INPUT_PD << 16) | (INPUT_PD << 8) | INPUT_PU; // P0.7_P0.4
  PORT0->IOCR8 = (INPUT_PU << 24) | (INPUT_PU << 16) | (INPUT_PD << 8) | INPUT_PD; // P0.11_P0.8
  PORT0->IOCR12 =(INPUT_PD << 24) | (INPUT_PD << 16) | (INPUT_PU << 8) | INPUT_PU; // P0.15_P0.12

  // LED control ports configuration
  P1_0_set_mode(OUTPUT_PP_GP);
  P1_1_set_mode(OUTPUT_PP_GP);

  // Turn LED1 on, LED2 off
  LED1_ON;
  LED2_OFF;

  // UART initialization
  UART_Init();

  // System Timer initialization
  SysTick_Config(SystemCoreClock / SYS_TICK_FRQ);

  // Sampling timer initialization
  SamplingTimer_Init();

  // The main loop
  while(1){

    // UART receiver service
    if ((USIC0_CH0->TRBSR & USIC_CH_TRBSR_REMPTY_Msk) == 0){         // if receive buffer not empty
      cmd_buf.data[cmd_buf.len++] = (USIC0_CH0->OUTR & 0x000000FF);  // read data from receiver FIFO
      rx_timer = 1;                                                  // start receive timeout timer
    }

    // UART receiver timeout service
    if (rx_timer >= RX_TIMEOUT){                           // if receiver timeout
      rx_timer = 0;                                        // stop timer
      cmd_buf.len = 0;                                     // clear receive command buffer
    }

    // SUMP command parser
    if (cmd_buf.len != 0){                                 // if data in command buffer
      switch (cmd_buf.data[0]){                            // analyze command
        // SUMP short commands (length 1B)
        case SUMP_ID             : // request for the device identification
        case SUMP_GET_MDATA      : // get metadata
        case SUMP_RESET          : // reset the device
        case SUMP_RUN            : // arm the trigger
        case SUMP_TEST           : // trigger a self test pattern
        case SUMP_STOP           : // finish the current capture and dump the samples
        case SUMP_XON            : // put transmitter out of pause mode
        case SUMP_XOFF           : // put transmitter in pause mode
                                   SUMP_ShortCmd();        // execute command
                                   cmd_buf.len = 0;        // clear receive command buffer
                                   break;

        // SUMP long commands (length 5B)
        case SUMP_SET_DVDR       : // set divider
        case SUMP_SET_TRG_MSK_S0 : // set trigger mask for stage 0
        case SUMP_SET_TRG_MSK_S1 : // set trigger mask for stage 1
        case SUMP_SET_TRG_MSK_S2 : // set trigger mask for stage 2
        case SUMP_SET_TRG_MSK_S3 : // set trigger mask for stage 3
        case SUMP_SET_TRG_VAL_S0 : // set trigger value for stage 0
        case SUMP_SET_TRG_VAL_S1 : // set trigger value for stage 1
        case SUMP_SET_TRG_VAL_S2 : // set trigger value for stage 2
        case SUMP_SET_TRG_VAL_S3 : // set trigger value for stage 3
        case SUMP_SET_TRG_CFG_S0 : // set trigger configuration for stage 0
        case SUMP_SET_TRG_CFG_S1 : // set trigger configuration for stage 1
        case SUMP_SET_TRG_CFG_S2 : // set trigger configuration for stage 2
        case SUMP_SET_TRG_CFG_S3 : // set trigger configuration for stage 3
        case SUMP_SET_RD_CNT     : // set read and delay count
        case SUMP_SET_FLAGS      : // set flags
                                   if (cmd_buf.len == 5){  // when complete data field received
                                     SUMP_LongCmd();       // execute command
                                     cmd_buf.len = 0;      // clear receive command buffer
                                   }
                                   break;

        default :                  // unknown command
                                   cmd_buf.len = 0;        // clear receive command buffer
                                   break;
      }
    }

    // Input samples transfer
    if (sample_tx.state == ON){                            // if transfer to SUMP client
      if (sample_buf.wr_idx >= sample_tx.cntr){            // if sample buffer was not rounded
        UART_SendData(&sample_buf.data[sample_buf.wr_idx - sample_tx.cntr], sample_tx.cntr);
      }
      else {                                               // if sample buffer was rounded
        UART_SendData(&sample_buf.data[SAMPLE_BUF_SIZE + sample_buf.wr_idx - sample_tx.cntr],
                       sample_tx.cntr - sample_buf.wr_idx);
        UART_SendData(&sample_buf.data[0], sample_buf.wr_idx);
      }
    sample_tx.state = OFF;                                 // samples transfer completed
    }
  }
}

/**-----------------------------------------------------------------------------
  Name         :  UART_Init
  Description  :  Initialization of the UART
                    UART I/O pins:    P2.1 (TxD), P2.2 (RxD)
                    UART peripheral:  USIC0, channel 0
                    UART speed:       115200bps
                    UART data format: 8N1
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void UART_Init(void)
{
  // Configuration of USIC0 system clock
  // - clear USIC0 clock gating
  SCU_GENERAL->PASSWD = 0x000000C0UL;                      // disable bit protection
  SCU_CLOCK->CGATCLR0 |= SCU_CLOCK_CGATCLR0_USIC0_Msk;     // clear gating of USIC0 clock
  SCU_GENERAL->PASSWD = 0x000000C3UL;                      // enable bit protection
  // - enable USIC0 kernel clock and USIC0 functionality
  USIC0_CH0->KSCFG |= USIC_CH_KSCFG_MODEN_Msk | USIC_CH_KSCFG_BPMODEN_Msk;

  // Configuration of the baud rate generator
  // UART transmition clock is derived from system peripheral clock fBP. In the first step
  // system peripheral clock is divided by fractional divider producing fFD frequency:
  //
  //                           fFD = fPB * STEP/1024
  // where:  fFD  - fractional divider output frequency
  //         fPB  - system peripheral clock (fPB = 32MHz)
  //         SETP - fractional divider division step
  //
  // In the second step fractional divider output frequency is forwarded to divide mode
  // counter as fPIN input freqency. UART transmission clock frequency equals to:
  //
  //     fASC = fPIN/((PPPEN+1)*(CTQSEL)) * 1/(PDIV + 1) * 1/(DCTQ + 1) * 1/(PCTQ + 1)
  // where:  fASC  - UART transmission clock frequency
  //         fPIN  - divide mode counter input frequency (fPIN = fFD)
  //         PPPEN - input fPIN divider by 2
  //         CTQSEL- length of a time quantum (2 if CQTSEL=10b, 1 if CQTSEL=00b)
  //         PDIV  - divider Factor to Generate fPDIV
  //         DCTQ  - denominator for time quanta counter
  //         PCTQ  - pre-divider for time quanta counter

  // - fractional divider configuration
  //     - divider mode: fractional divider (DM=10b)
  //     - division step: STEP=UART_FDR_STEP
  USIC0_CH0->FDR &= ~(USIC_CH_FDR_DM_Msk | USIC_CH_FDR_STEP_Msk);
  USIC0_CH0->FDR |= (0x02UL << USIC_CH_FDR_DM_Pos) | (UART_FDR_STEP << USIC_CH_FDR_STEP_Pos);

  // - baud rate generator configuration
  //     - shift clock output configuration: passive level 0, delay disabled (SCLKCFG=00b)
  //     - master clock configuration: passive level 0 (MCLKCFG=0)
  //     - shift clock output: SCLK from the baud rate generator (SCLKOSEL=0)
  //     - divider factor to generate fPDIV: PDIV=UART_BRG_PDIV
  //     - denominator for time quanta counter: DCTQ=UART_BRG_DCTQ
  //     - pre-divider for time quanta counter: PCTQ=UART_BRG_PCTQ
  //     - length of a time quantum: fCTQIN = fPDIV (CTQSEL = 00b)
  //     - 1:2 fPPP divider disabled / fPPP=fPIN (PPPEN=0)
  //     - timing measurement disabled (TMEN=0)
  //     - clock selection: fractional divider frequency / fPIN = fFD (CLKSEL=00b)
  USIC0_CH0->BRG &= ~(USIC_CH_BRG_CLKSEL_Msk | USIC_CH_BRG_PPPEN_Msk |
                      USIC_CH_BRG_PDIV_Msk | USIC_CH_BRG_DCTQ_Msk | USIC_CH_BRG_PCTQ_Msk);
  USIC0_CH0->BRG |= (UART_BRG_PDIV << USIC_CH_BRG_PDIV_Pos) |
                    (UART_BRG_DCTQ << USIC_CH_BRG_DCTQ_Pos) |
                    (UART_BRG_PCTQ << USIC_CH_BRG_PCTQ_Pos);

  // Configuration of USIC shift control
  //   - word length: 8 bits (WLE=7)
  //   - frame lenght: 8 bits (FLE=7)
  //   - transmission mode: control signal active when high (TRM=01b)
  //   - data output configuration: data non inverted (DOCFG=00b)
  //   - port control direction: HWDcntrl pins are inputs (HPCDIR=0)
  //   - data shift mode: one bit at time (DSM=00b)
  //   - passive data level: high (PDL=1)
  //   - shift direction: LSB first (SDIR=0)
  USIC0_CH0->SCTR &= ~(USIC_CH_SCTR_WLE_Msk | USIC_CH_SCTR_FLE_Msk | USIC_CH_SCTR_TRM_Msk);
  USIC0_CH0->SCTR |= (0x07UL << USIC_CH_SCTR_WLE_Pos) |
                     (0x07UL << USIC_CH_SCTR_FLE_Pos) |
                     (0x01UL << USIC_CH_SCTR_TRM_Pos) |
                     USIC_CH_SCTR_PDL_Msk;

  // Configuration of USIC Transmit Control/Status Register
  //   - TBUF data enable: transmission starts at TDV=1 (TDEN=01b)
  //   - TBUF data single shot mode: allow word-by-word data transmission
  //     which avoid sending the same data several times (TDSSM=1)
  USIC0_CH0->TCSR &= ~(USIC_CH_TCSR_TDEN_Msk);
  USIC0_CH0->TCSR |= (0x01UL << USIC_CH_TCSR_TDEN_Pos) | USIC_CH_TCSR_TDSSM_Msk;

  // Configuration of protocol
  //   - master clock enable: disabled (MCLK=0)
  //   - transmitter status enable: PSR[9] flag not modified (TSTEN=0)
  //   - receiver status enable: PSR[9] flag not modified (RSTEN=0)
  //   - pulse length: pulse length equal to bit length (PL=000b)
  //   - sample point: 8 (SP=8)
  //   - frame finish interrupt enable: int disabled (FFIEN=0)
  //   - format error interrupt enable: int disabled (FEIEN=0)
  //   - receiver noise detection interrupt enable: int disabled (RNIEN=0)
  //   - colision detection enable: disabled (CDEN=0)
  //   - synchronization break interrupt enable: int disabled (SBIEN=0)
  //   - idle detection mode: bus idle detecton switched off (IDM=0)
  //   - stop bit: 1 (STPB=0)
  //   - sample mode: three samples per bit (SMD=1)
  USIC0_CH0->PCR &= ~(USIC_CH_PCR_ASCMode_PL_Msk | USIC_CH_PCR_ASCMode_SP_Msk | USIC_CH_PCR_ASCMode_STPB_Msk);
  USIC0_CH0->PCR |= (8UL << USIC_CH_PCR_ASCMode_SP_Pos) | USIC_CH_PCR_ASCMode_SMD_Msk;

  // Configuration of transmit buffer
  //   - transmit buffer error and standard interrupts disabled (TBERIEN=0, STBIEN=0)
  //   - buffer event on limit overflow (LOF=0)
  //   - buffer size: 32 entries (SIZE=101b)
  //   - alternative and standard node pointers: not used (ATBINP=xxxb, STBINP=xxxb)
  //   - standard trigger through STBT bit enable: disabled (STBTEN=0)
  //   - standard trigger mode: 0 (STBTM=0)
  //   - limit for interrupt generation: not used (LIMIT=xxxxxxb)
  //   - data pointer: 0 (DPTR=000000b)
  USIC0_CH0->TBCTR &= ~(USIC_CH_TBCTR_SIZE_Msk | USIC_CH_TBCTR_DPTR_Msk);
  USIC0_CH0->TBCTR |= ((0x05UL << USIC_CH_TBCTR_SIZE_Pos)|(0x00 << USIC_CH_TBCTR_DPTR_Pos));

  // Configuration of receive buffer
  //   - receive buffer error interrupt disabled (RBERIEN=0)
  //   - receive buffer standard interrupt disabled (SRBIEN=0)
  //   - alternative interrupt enable: disabled (ARBIEN=0)
  //   - buffer event on limit overflow (LOF=0)
  //   - receiver notification mode: RCI mode (RNM=1)
  //   - buffer size: 32 entries (SIZE=101b)
  //   - receiver control information mode: 0&WLEN (RCIM=10b)
  //   - alternative node pointer: not used (ARBINP=xxxb)
  //   - standard node pointer: output SR0 (SRBINP=000b)
  //   - standard trigger through SRBT bit enable: disabled (SRBTEN=0)
  //   - standard trigger mode: 0 (SRBTM=0)
  //   - limit for interrupt generation: not used (LIMIT=xxxxxxb)
  //   - data pointer: 32 (DPTR=100000b)
  USIC0_CH0->RBCTR &= ~(USIC_CH_RBCTR_SIZE_Msk | USIC_CH_RBCTR_RCIM_Msk |
                        USIC_CH_RBCTR_SRBINP_Msk | USIC_CH_RBCTR_DPTR_Msk);
  USIC0_CH0->RBCTR |= (USIC_CH_RBCTR_RNM_Msk | (0x05UL << USIC_CH_RBCTR_SIZE_Pos) |
                      (0x02UL << USIC_CH_RBCTR_RCIM_Pos) | (32UL << USIC_CH_RBCTR_DPTR_Pos));

  // Configuration of channel control register
  //   - all interrupts disabled (BRGIEN=0, AIEN=0, RIEN=0, TBIEN=0, TSIEN=0, DLIEN=0, RSIEN=0)
  //   - parity mode: parity generation disabled (PM=00b)
  //   - hardware port control: disabled (HPCEN=00b)
  //   - operating mode: ASC(SCI, UART) protocol (MODE=0010b)
  USIC0_CH0->CCR &= ~(USIC_CH_CCR_PM_Msk | USIC_CH_CCR_MODE_Msk);
  USIC0_CH0->CCR |= 0x02UL << USIC_CH_CCR_MODE_Pos;

  // Configuration of TxD and RxD pins
  // - configuration of port P2.1 as TxD
  P2_1_set_mode(OUTPUT_PP_AF6);                            // set P2.1 as alternative function ALT6 (U0C0.DOUT0) output
  P2_1_enable_digital();                                   // switch analog pin to digital

  // - configuration of port P2.2 as RxD
  P2_2_set_mode(INPUT);                                    // set P2.2 as input
  USIC0_CH0->DX3CR &= ~(USIC_CH_DX3CR_DSEL_Msk);           // select P2.2 as input for USIC DX3 -> UCIC DX0
  USIC0_CH0->DX0CR &= ~(USIC_CH_DX0CR_DSEL_Msk);           // \ route USIC DX3 input to
  USIC0_CH0->DX0CR |= 6UL << USIC_CH_DX0CR_DSEL_Pos;       // /   USIC DX0 (DSEL=DX0G)
  P2_2_enable_digital();                                   // switch analog pin to digital

  // Interrupt configuration
  //   - interrupt line: SR0 (RBCTR.SRBINP=000b)
  //   - interrupt source: receive buffer event on OUTR updating (RBCTR.RNM=1, RBCTR.RCIM=10b)
  //   - receive buffer standard interrupt: disabled (RBCTR.SRBIEN=0) (it will be enabled when in need)
  //   - interrupt priority (NVIC): the highest
  //   - interrupt enable (NVIC): enabled
  NVIC_SetPriority(USIC0_0_IRQn, 0x00);
  NVIC_EnableIRQ(USIC0_0_IRQn);
}

/**-----------------------------------------------------------------------------
  Name         :  UART_SendData
  Description  :  Uploading data to uart transmit buffer
  Argument(s)  :  p_data - pointer to data to be sent
                  len    - data length
  Return value :  none
------------------------------------------------------------------------------*/
void UART_SendData(uint8_t *p_data, uint16_t len)
{
  while (len != 0){                    // if there is data to be sent
    while ((USIC0_CH0->TRBSR & USIC_CH_TRBSR_TFULL_Msk) != 0);   // wait for free space in transmit buffer
    USIC0_CH0->IN[0] = *p_data;        // write data to transmit buffer
    p_data ++;                         // address the next data cell
    len --;                            // count data left to send
  }
}

/**-----------------------------------------------------------------------------
  Name         :  SamplingTimer_Init
  Description  :  Initialization of sampling timer
                  When enabled, sampling timer generates interrupts with defined
                  sampling frequency. The timer is emulted by slice CC40 of the CCU40
                  module working in free running mode. The slice CC40 is clocked by
                  PCLK and generates interrupt every period match.
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void SamplingTimer_Init(void)
{
  // Configuration of CCU40 system clock
  // - clear CCU40 clock gating
  SCU_GENERAL->PASSWD = 0x000000C0UL;                      // disable bit protection
  SCU_CLOCK->CGATCLR0 |= SCU_CLOCK_CGATCLR0_CCU40_Msk;     // clear gating of CCU40 clock
  SCU_GENERAL->PASSWD = 0x000000C3UL;                      // enable bit protection

  // Prescaler enable
  CCU40->GIDLC |= CCU4_GIDLC_SPRB_Msk;

  // Configuration of global control register
  //   - multi channel transfer shadow request: not used (MSDE=xxb)
  //   - slice 3/2/1/0 multi channel shadow transfer enable: shadow transfer request
  //     only by software (MSE3=MSE2=MSE1=MSE0=0)
  //   - suspend mode configuration: request ignored (SUSCFG=00b)
  //   - prescaler input clock: module clock (PCIS=00b)
  //   - prescaler clear: software only (PRBC=000b)
  CCU40->GCTRL &= ~(CCU4_GCTRL_PCIS_Msk);

  // Configuration of CC40 timer slice
  //   - multi channel mode: disabled (MCME=0)
  //   - external modulation type: not used (EMT=x)
  //   - external modulation synchronization: not used (EMS=x)
  //   - TRAP state clear control: automatic exit (TRPSW=0)
  //   - TRAP synchronization enable: exit not synchronized (TRPSE=0)
  //   - TRAP enable: no affecton on output (TRAPE=0)
  //   - floating prescaler enable: disabled (FPE=0)
  //   - dither input: not used (DIM=x)
  //   - dither enable: disabled (DITHE=00b)
  //   - continous capture enable: not used (CCS=x)
  //   - equal capture event enable: not used (SCE=x)
  //   - extended start control: not used (STRM=x)
  //   - extended stop control: not used (ENDM=xxb)
  //   - clear on capture control: not used (CAPC=xxb)
  //   - extended capture mode: not used (ECM=x)
  //   - capture compare mode: compare mode (CMOD=0)
  //   - shadow transfer on clear: enabled (CLST=0)
  //   - timer single shot mode: disabled (TSSM=0)
  //   - timer counting mode: edge aligned (TCM=0)
  CCU40_CC40->TC = 0x00000000;

  // Configuration of timer prescaler (for test purposes, prescaler set prior to inputs capture)
  //   - prescaler mode: normal mode (FPE=0)
  //   - prescaler value: 1 (PSIV=0000b) or 256 (PSIV=1000b)
  //CCU40_CC40->PSC = 0x08;

  // Configuration of timer period (for test purposes, period set prior to inputs capture)
  //CCU40_CC40->TIMER = 0x0000;          // timer clear
  //CCU40_CC40->PRS = 32000-1;           // test period 100ms
  //CCU40->GCSS |= CCU4_GCSS_S0SE_Msk;   // load period, compare and passive level shadow regs.

  // Interrupt configuration
  //   - interrupt line: CC40SR0 (CC40SRS.POSR=00b)
  //   - interrupt source: period match while counting up event (CC40INTE.PME=1)
  //   - interrupt priority (NVIC): high
  //   - interrupt enable (NVIC): enabled
  CCU40_CC40->SRS &= ~(CCU4_CC4_SRS_POSR_Msk);
  CCU40_CC40->INTE |= CCU4_CC4_INTE_PME_Msk;

  NVIC_SetPriority(CCU40_0_IRQn, 0x01);
  NVIC_EnableIRQ(CCU40_0_IRQn);

  // Enable of CC40 timer slice
  CCU40->GIDLC |= CCU4_GIDLC_CS0I_Msk;

 // Start CC40 timer (for test purposes, timer enabled prior to inputs capture)
 //CCU40_CC40->TCSET = CCU4_CC4_TCSET_TRBS_Msk;
}

/**-----------------------------------------------------------------------------
  Name         :  SUMP_ShortCmd
  Description  :  Execution of the SUMP short command
  Argument(s)  :  command code in cmd_buf.data[0] (global var)
  Return value :  none
------------------------------------------------------------------------------*/
void SUMP_ShortCmd(void)
{
  switch (cmd_buf.data[0]){    // analyze command code

    case SUMP_ID             : // request for the device identification
                               UART_SendData((uint8_t *)sla_stdID, sizeof sla_stdID);
                               break;

    case SUMP_GET_MDATA      : // get metadata
                               // - send device name
                               UART_SendData((uint8_t *)sla_devName, sizeof sla_devName);
                               // - send firmware version
                               UART_SendData((uint8_t *)sla_devVer, sizeof sla_devVer);
                               // - send available sample memory
                               UART_SendData((uint8_t *)sla_memSize, sizeof sla_memSize);
                               // - send maksimum sample rate
                               UART_SendData((uint8_t *)sla_smplRate, sizeof sla_smplRate);
                               // - send number of usable probes
                               UART_SendData((uint8_t *)sla_probesQtty, sizeof sla_probesQtty);
                               // - send protocol version
                               UART_SendData((uint8_t *)sla_protVer, sizeof sla_protVer);
                               // - send meta data end key
                               UART_PUTCH(MDATA_END);
                               break;

    case SUMP_RESET          : // reset the device
                               // - stop sampling timer
                               CCU40_CC40->TCCLR = CCU4_CC4_TCCLR_TRBC_Msk;
                               // - switch off LED2
                               LED2_OFF;
                               // - disable UART RxBuf irq
                               USIC0_CH0->RBCTR &= ~USIC_CH_RBCTR_SRBIEN_Msk;
                               // - start SysTick Timer
                               SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;
                               break;

    case SUMP_RUN            : // arm the trigger
                               // - stop SysTick Timer
                               SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
                               // - switch off LED1
                               LED1_OFF;
                               // - enable UART RxBuf interrupt
                               USIC0_CH0->RBCTR |= USIC_CH_RBCTR_SRBIEN_Msk;
                               // - switch on LED2
                               LED2_ON;
                               // - init sampling buffer
                               sample_buf.wr_idx = 0;
                               // - init samples transmitter
                               sample_tx.state = OFF;
                               // - clear sampling timer
                               CCU40_CC40->TCCLR = CCU4_CC4_TCCLR_TCC_Msk;
                               // - start sampling timer
                               CCU40_CC40->TCSET = CCU4_CC4_TCSET_TRBS_Msk;
                               break;

// Unimplemented SUMP short commands
//    case SUMP_TEST           : // trigger a self test pattern
//    case SUMP_STOP           : // finish the current capture and dump the samples
//    case SUMP_XON            : // put transmitter out of pause mode
//    case SUMP_XOFF           : // put transmitter in pause mode
  }
}

/**-----------------------------------------------------------------------------
  Name         :  SUMP_LongCmd
  Description  :  Execution of the SUMP long command
  Argument(s)  :  command code in cmd_buf.data[0]  (global var)
                  command data in cmd_buf.data[1]..cmd_buf.data[4]  (global var)
  Return value :  none
------------------------------------------------------------------------------*/
void SUMP_LongCmd(void)
{
  switch (cmd_buf.data[0]){    // analyze command code

    case SUMP_SET_DVDR       : // set divider for sampling frequency fsmpl = fclk/(dvdr+1)
                               //       dvdr = cmd_buf.data[4]..cmd_buf.data[1]
                               // Because fclk=32MHz, divider value ranges from 63 for fsmpl=0,5MHz
                               // to 3199999 for fsmpl=10Hz. Thus MSB of dvdr is always 0
                               // (i.e. cmd_buf.data[4]=0)
                               if (cmd_buf.data[3]){       // when dvdr > 0xFFFF
                                 // - set CC40 slice prescaler to 256
                                 CCU40_CC40->PSC = 0x08;
                                 // - set CC40 slice period
                                 CCU40_CC40->PRS = (cmd_buf.data[3] << 8) + cmd_buf.data[2];
                               }
                               else {                      // when dvdr =< 0xFFFF
                                 // - set CC40 slice prescaler to 1
                                 CCU40_CC40->PSC = 0x00;
                                 // - set CC40 slice period (min. 63)
                                 CCU40_CC40->PRS = (cmd_buf.data[2] << 8) + cmd_buf.data[1];
                                 if (CCU40_CC40->PRS < 63) CCU40_CC40->PRS = 63;
                               }

                               // Load timer period, compare and passive level shadow regs.
                               CCU40->GCSS |= CCU4_GCSS_S0SE_Msk;
                               break;

    case SUMP_SET_TRG_MSK_S0 : // set trigger mask for stage 0
                               sample_buf.trig_mask = cmd_buf.data[1];
                               break;

    case SUMP_SET_TRG_VAL_S0 : // set trigger value for stage 0
                               sample_buf.trig_value = cmd_buf.data[1];
                               break;

    case SUMP_SET_RD_CNT     : // set read and delay count
                               // - read count
                               sample_tx.cntr = ((cmd_buf.data[2]<<8) + cmd_buf.data[1] + 1)<<2;
                               // - delay count (trigger count)
                               sample_buf.trig_cntr = (((cmd_buf.data[4]<<8) + cmd_buf.data[3] + 1)<<2) + 1;
                               break;

// Unimplemented SUMP long commands
//    case SUMP_SET_TRG_MSK_S1 : // set trigger mask for stage 1
//    case SUMP_SET_TRG_MSK_S2 : // set trigger mask for stage 2
//    case SUMP_SET_TRG_MSK_S3 : // set trigger mask for stage 3
//    case SUMP_SET_TRG_VAL_S1 : // set trigger value for stage 1
//    case SUMP_SET_TRG_VAL_S2 : // set trigger value for stage 2
//    case SUMP_SET_TRG_VAL_S3 : // set trigger value for stage 3
//    case SUMP_SET_TRG_CFG_S0 : // set trigger configuration for stage 0
//    case SUMP_SET_TRG_CFG_S1 : // set trigger configuration for stage 1
//    case SUMP_SET_TRG_CFG_S2 : // set trigger configuration for stage 2
//    case SUMP_SET_TRG_CFG_S3 : // set trigger configuration for stage 3
//    case SUMP_SET_FLAGS      : // set flags
  }
}

/**-----------------------------------------------------------------------------
  Name         :  SysTickHandler
  Description  :  SysTick Timer Interrupt Handler
                  Handler services LED1 and UART rx timeout timer
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void SysTick_Handler(void)
{
  static uint32_t led_evnt_cntr = 0;

  // UART rx timer service
  if (rx_timer != 0){                  // if rx timer active
    rx_timer ++;                       // count time since the last received character
  }

  led_evnt_cntr++;                     // count LED events
  if (led_evnt_cntr >= LED_EVENT){     // if it's time to service LED
    LED1_TOGGLE;
    led_evnt_cntr = 0;                 // reset LED event counter
  }
}

/**-----------------------------------------------------------------------------
  Name         :  USIC0_0_IRQHandler
  Description  :  UART Rx interrupt handler
                  Handler stops capturing inputs
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void USIC0_0_IRQHandler(void)
{
  USIC0_CH0->RBCTR &= ~USIC_CH_RBCTR_SRBIEN_Msk;           // disable UART RxBuf irq
  CCU40_CC40->TCCLR = CCU4_CC4_TCCLR_TRBC_Msk;             // stop sampling timer
  LED2_OFF;                                                // switch off LED2
  SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;                // start SysTick Timer
}

/**-----------------------------------------------------------------------------
  Name         :  CCU40_0_IRQHandler
  Description  :  Sampling timer interrupt handler
                  Handler captures inputs and stores their states in ring buffer.
                  Next, it checks trigger condition and counts number of captured
                  samples.
  Argument(s)  :  none
  Return value :  none
------------------------------------------------------------------------------*/
void CCU40_0_IRQHandler(void)
{
  uint32_t in_sample;                                      // input sample

  // Sample inputs
  in_sample = PORT0->IN;                                   // read input port P0
  in_sample ^= (in_sample >> 13);                          // b2b1b0 = /P0.15/P0.14/P0.0
  in_sample ^= (in_sample << 10);                          // b12b11b10 = P0.15 P0.14 P0.0
  in_sample >>= 5;                                         // b7..b0 = P0.15P0.14P0.0P0.9P0.8P0.7P0.6P0.5

  // Store inputs sample in buffer
  sample_buf.data[sample_buf.wr_idx] = in_sample;          // store input sample in buffer
  sample_buf.wr_idx ++;                                    // index the next free position in buffer
  sample_buf.wr_idx &= (SAMPLE_BUF_SIZE - 1);              // if buffer overflow, start from the beginning

  // Service trigger
  if (sample_buf.trig_mask != 0){                          // if sampling not triggered
    // Check trigger condition
    if ((in_sample & sample_buf.trig_mask) == sample_buf.trig_value){
      sample_buf.trig_mask = 0;                            // trigger released
    }
  }
  else {                                                   // if sampling triggered
    sample_buf.trig_cntr --;                               // count samples after trigger
    if (sample_buf.trig_cntr == 0){                        // if specified number of samples has been reached
      USIC0_CH0->RBCTR &= ~USIC_CH_RBCTR_SRBIEN_Msk;       // disable UART RxBuf irq
      CCU40_CC40->TCCLR = CCU4_CC4_TCCLR_TRBC_Msk;         // stop sampling timer
      NVIC_ClearPendingIRQ(CCU40_0_IRQn);                  // clear pending timer irq
      LED2_OFF;                                            // switch off LED2
      SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;            // start SysTick Timer
      sample_tx.state = ON;                                // activate samples transfer
    }
  }
}
